/// A dynamic query structure.
pub trait DynamicQuery {
    type Definition;

    fn filter_ref(&mut self, def: &Self::Definition);

    fn filter(mut self, def: &Self::Definition) -> Self
    where
        Self: Sized,
    {
        self.filter_ref(def);
        self
    }
}

mod macros;

#[cfg(test)]
mod tests {
    use serde::{Deserialize, Serialize};

    // just tests that the macro compiles
    crate::query_def! {
        /// Foo doc.
        Foo(FooDef) {
            /// Field doc.
            #[serde(flatten)]
            bar: (Bar),
            baz: (bool),
            bars: 'vec (Bar),
            bars_by_bazs: 'map (bool, Bar),
        }

        /// Bar doc.
        Bar(BarDef) {
            baz_inner: (bool),
        }
    }

    // make sure A. serde is working and B. we get expected structure
    #[test]
    fn test_json() {
        assert_eq!(serde_json::to_string(&Foo {
            bar: Some(Bar {
                baz_inner: Some(true),
            }),
            baz: Some(false),
            bars: Some(vec![]),
            bars_by_bazs: Some(std::collections::HashMap::new()),
        }).ok(), Some(r#"{"baz_inner":true,"baz":false,"bars":[],"bars_by_bazs":{}}"#.to_owned()));
    }
}
